import React from 'react';
import { CompositeLayer } from 'deck.gl';
import { ScatterplotLayer } from '@deck.gl/layers';
import { getProperty } from '../../../utlis';
import NodeOptions from './NodeOptions';
import { Node, LayerProps } from '../../../mapvis.types';
import { color, scaleLinear, scaleOrdinal } from 'd3';
import { dataColors, visualizationColors } from 'config';

export const NodeLayerConfig: {
  collisionFilter: boolean;
  opacity: number;
  fillColor: {
    filled: boolean;
    type: string;
    label: string;
    accessor: string;
    fixed: boolean;
    defaultValue: string;
    value: string;
    scaleRange: string[];
    condition: string;
  };
} = {
  collisionFilter: true,
  opacity: 1,
  fillColor: {
    filled: true,
    type: 'rgb',
    label: 'color',
    accessor: 'getFillColor',
    fixed: true,
    defaultValue: visualizationColors.GPPrimary.colors[1][0],
    value: visualizationColors.GPPrimary.colors[1][0],
    scaleRange: visualizationColors.GPPrimary.colors[1],
    condition: '',
  },
};

export class NodeLayer extends CompositeLayer<LayerProps> {
  static type = 'Node';
  static layerOptions = NodeLayerConfig;

  static generateLayerOptions(layer: any, updatedLayer: void, graphInfo: { [key: string]: any }, deleteLayer: void) {
    return <NodeOptions layer={layer} updatedLayer={updatedLayer} graphInfo={graphInfo} deleteLayer={deleteLayer} />;
  }

  updateState({ props, oldProps, context, changeFlags }: { props: any; oldProps: any; context: any; changeFlags: any }) {
    const { graph, config } = props;

    if (!config.fillColor.fixed) {
      const colorAttribute = graph.getGraphInfo().nodeAttributes[config.fillColor.condition];
      const colorScale = this.createColorScale(config.fillColor, colorAttribute);
      this.setState({ colorScale });
    }
  }

  createColorScale(colorProp: { [key: string]: any }, colorAttribute: { [key: string]: any }) {
    if (colorAttribute.dataType == 'boolean') {
      return scaleOrdinal().domain(['0', '1']).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'number') {
      return scaleLinear().domain(colorAttribute.values).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'string') {
      return scaleOrdinal()
        .domain([...colorAttribute.values])
        .range(colorProp.scaleRange);
    } else {
      return null;
    }
  }

  getColor(node: Node, colorProps: { [key: string]: any }) {
    if (colorProps.fixed) {
      return colorProps.value !== undefined ? colorProps.value : colorProps.defaultValue;
    } else {
      const { condition } = colorProps;
      if (!condition) {
        return colorProps.defaultValue;
      }
      const value = getProperty(node.attributes, condition);
      return (this.state as any).colorScale(value);
    }
  }

  renderLayers() {
    const { graph, config, visible, getNodeLocation } = this.props;

    return new ScatterplotLayer({
      hidden: visible,
      data: graph.nodes,
      pickable: true,
      opacity: config.opacity,
      filled: config.fillColor.filled,
      radiusScale: 6,
      radiusMinPixels: 7,
      radiusMaxPixels: 100,
      lineWidthMinPixels: 1,
      getPosition: (d: any) => getNodeLocation(d.id),
      // getFillColor: (d: any) => this.getColor(d, config.fillColor),
      getFillColor: (d: any) => {
        const c = color(config.fillColor.value as string)?.rgb();
        if (!c) {
          return [0, 0, 0];
        }
        return [c.r, c.g, c.b];
      },
      getRadius: (d: any) => 5,
    });
  }
}

NodeLayer.layerName = 'Node';
